@@ -23,6 +23,7 @@
 #include <gdk/gdktypes.h>
 #include <gtk/gtkhbox.h>
 #include <bonobo/bonobo-socket.h>
+#include <bonobo/bonobo-exception.h>
 
 enum {
 	ACTIVATED,
@@ -203,13 +204,14 @@ bonobo_control_frame_set_remote_window (GtkWidget          *socket,
 	 * If we are not yet bound to a remote control, don't do
 	 * anything.
 	 */
-	if (control == CORBA_OBJECT_NIL)
+	if (control == CORBA_OBJECT_NIL || !socket)
 		return;
 
 	/*
 	 * Sync the server, since the XID may have been created on the
 	 * client side without communication with the X server.
 	 */
+	/* FIXME: not very convinced about this flush */
 	gdk_flush ();
 
 	/*
@@ -217,7 +219,8 @@ bonobo_control_frame_set_remote_window (GtkWidget          *socket,
 	 * remote Control.
 	 */
 	CORBA_exception_init (&ev);
-	id = bonobo_control_windowid_from_x11 (GDK_WINDOW_XWINDOW (socket->window));
+	id = bonobo_control_windowid_from_x11 (
+		GDK_WINDOW_XWINDOW (socket->window));
 
 	Bonobo_Control_setWindowId (control, id, &ev);
 	g_free (id);
@@ -227,7 +230,7 @@ bonobo_control_frame_set_remote_window (GtkWidget          *socket,
 }
 
 static void
-bonobo_control_frame_create_socket (BonoboControlFrame  *control_frame)
+bonobo_control_frame_create_socket (BonoboControlFrame *control_frame)
 {
 	/*
 	 * Now create the GtkSocket which will be used to embed
@@ -236,6 +239,10 @@ bonobo_control_frame_create_socket (BonoboControlFrame  *control_frame)
 	control_frame->priv->socket = bonobo_socket_new ();
 	gtk_widget_show (control_frame->priv->socket);
 
+	bonobo_socket_set_control_frame (
+		BONOBO_SOCKET (control_frame->priv->socket),
+		control_frame);
+
 	/*
 	 * Connect to the focus events on the socket so
 	 * that we can provide the autoactivation feature.
@@ -278,15 +285,6 @@ bonobo_control_frame_create_socket (BonoboControlFrame  *control_frame)
 	gtk_box_pack_start (GTK_BOX (control_frame->priv->container),
 			    control_frame->priv->socket,
 			    TRUE, TRUE, 0);
-
-	/*
-	 * When the socket is realized, we pass its Window ID to our
-	 * Control.
-	 */
-	gtk_signal_connect (GTK_OBJECT (control_frame->priv->socket),
-			    "realize",
-			    GTK_SIGNAL_FUNC (bonobo_control_frame_set_remote_window),
-			    control_frame);
 }
 				
 
@@ -368,15 +366,18 @@ bonobo_control_frame_destroy (GtkObject *object)
 {
 	BonoboControlFrame *control_frame = BONOBO_CONTROL_FRAME (object);
 
+	gtk_widget_destroy (control_frame->priv->container);
+
 	if (control_frame->priv->control != CORBA_OBJECT_NIL)
 		bonobo_object_release_unref (control_frame->priv->control, NULL);
 	control_frame->priv->control = CORBA_OBJECT_NIL;
 
-	gtk_widget_unref (control_frame->priv->container);
-
 	if (control_frame->priv->socket) {
-		gtk_signal_disconnect_by_data (GTK_OBJECT (control_frame->priv->socket),
-					       control_frame);
+		bonobo_socket_set_control_frame (
+			BONOBO_SOCKET (control_frame->priv->socket), NULL);
+		gtk_signal_disconnect_by_data (
+			GTK_OBJECT (control_frame->priv->socket),
+			control_frame);
 		gtk_widget_unref (control_frame->priv->socket);
 		control_frame->priv->socket = NULL;
 	}
@@ -386,6 +387,7 @@ bonobo_control_frame_destroy (GtkObject *object)
 	control_frame->priv->ui_container = CORBA_OBJECT_NIL;
 
 	g_free (control_frame->priv);
+	control_frame->priv = NULL;
 	
 	GTK_OBJECT_CLASS (bonobo_control_frame_parent_class)->destroy (object);
 }
@@ -401,6 +403,8 @@ bonobo_control_frame_activated (BonoboControlFrame *control_frame, gboolean stat
 
 /**
  * bonobo_control_frame_get_epv:
+ *
+ * Returns: The EPV for the default BonoboControlFrame implementation. 
  */
 POA_Bonobo_ControlFrame__epv *
 bonobo_control_frame_get_epv (void)
@@ -798,8 +802,8 @@ bonobo_control_frame_bind_to_control (BonoboControlFrame *control_frame, Bonobo_
 	 * Introduce ourselves to the Control.
 	 */
 	Bonobo_Control_setFrame (control,
-				  bonobo_object_corba_objref (BONOBO_OBJECT (control_frame)),
-				  &ev);
+				 bonobo_object_corba_objref (BONOBO_OBJECT (control_frame)),
+				 &ev);
 	if (ev._major != CORBA_NO_EXCEPTION)
 		bonobo_object_check_env (BONOBO_OBJECT (control_frame), control, &ev);
 	CORBA_exception_free (&ev);
@@ -814,10 +818,9 @@ bonobo_control_frame_bind_to_control (BonoboControlFrame *control_frame, Bonobo_
 	 * If the socket is realized, then we transfer the
 	 * window ID to the remote control.
 	 */
-
 	if (GTK_WIDGET_REALIZED (control_frame->priv->socket))
-		bonobo_control_frame_set_remote_window (control_frame->priv->socket,
-							control_frame);
+		bonobo_control_frame_set_remote_window (
+			control_frame->priv->socket, control_frame);
 }
 
 /**
@@ -928,3 +931,105 @@ bonobo_control_frame_get_control_property_bag (BonoboControlFrame *control_frame
 
 	return pbag;
 }
+
+void
+bonobo_control_frame_size_request (BonoboControlFrame *control_frame,
+				   int *desired_width, int *desired_height)
+{
+	CORBA_Environment ev;
+	CORBA_short width, height;
+
+	g_return_if_fail (control_frame != NULL);
+	g_return_if_fail (BONOBO_IS_CONTROL_FRAME (control_frame));
+	g_return_if_fail (control_frame->priv->control != CORBA_OBJECT_NIL);
+	g_return_if_fail (desired_width != NULL);
+	g_return_if_fail (desired_height != NULL);
+
+	CORBA_exception_init (&ev);
+
+	Bonobo_Control_getDesiredSize (control_frame->priv->control,
+				       &width, &height, &ev);
+
+	if (ev._major != CORBA_NO_EXCEPTION) {
+		bonobo_object_check_env (
+			BONOBO_OBJECT (control_frame),
+			(CORBA_Object) control_frame->priv->control, &ev);
+
+		width = height = 0;
+	}
+
+	*desired_width = width;
+	*desired_height = height;
+
+	CORBA_exception_free (&ev);
+}
+
+/*
+ *  These methods are used by the bonobo-socket to
+ * sync the X pipe with the CORBA connection.
+ */
+void
+bonobo_control_frame_sync_realize (BonoboControlFrame *frame)
+{
+	Bonobo_Control control;
+	CORBA_Environment ev;
+
+	g_return_if_fail (BONOBO_IS_CONTROL_FRAME (frame));
+	
+	if (!frame->priv || frame->priv->control == CORBA_OBJECT_NIL)
+		return;
+
+	control = frame->priv->control;
+
+	bonobo_control_frame_set_remote_window (
+		frame->priv->socket, frame);
+
+	/*
+	 * We sync here so that we make sure that if the XID for
+	 * our window is passed to another application, SubstructureRedirectMask
+	 * will be set by the time the other app creates its window.
+	 */
+	gdk_flush ();
+
+	if (control == CORBA_OBJECT_NIL)
+		return;
+
+	CORBA_exception_init (&ev);
+
+	Bonobo_Control_realize (control, &ev);
+	if (BONOBO_EX (&ev))
+		g_warning ("Exception on unrealize '%s'",
+			   bonobo_exception_get_text (&ev));
+
+	CORBA_exception_free (&ev);
+
+	gdk_flush ();
+}
+
+void
+bonobo_control_frame_sync_unrealize (BonoboControlFrame *frame)
+{
+	Bonobo_Control control;
+	CORBA_Environment ev;
+
+	if (!frame->priv || frame->priv->control == CORBA_OBJECT_NIL)
+		return;
+
+	control = frame->priv->control;
+
+	gdk_flush ();
+
+	if (control == CORBA_OBJECT_NIL)
+		return;
+
+	CORBA_exception_init (&ev);
+
+	Bonobo_Control_unrealize (control, &ev);
+	if (BONOBO_EX (&ev))
+		g_warning ("Exception on unrealize '%s'",
+			   bonobo_exception_get_text (&ev));
+
+	CORBA_exception_free (&ev);
+
+	gdk_flush ();
+}
