@@ -17,6 +17,7 @@
 #include <bonobo/bonobo-main.h>
 #include <bonobo/bonobo-plug.h>
 #include <bonobo/bonobo-control.h>
+#include <bonobo/bonobo-exception.h>
 #include <gdk/gdkprivate.h>
 #include <gtk/gtkbox.h>
 #include <gtk/gtkmain.h>
@@ -205,7 +206,8 @@ impl_Bonobo_Control_activate (PortableServer_Servant servant,
 			bonobo_control_auto_unmerge (control);
 	}
 
-	gtk_signal_emit (GTK_OBJECT (control), control_signals [ACTIVATE], (gboolean) activated);
+	if (control->priv->active != activated)
+		gtk_signal_emit (GTK_OBJECT (control), control_signals [ACTIVATE], (gboolean) activated);
 
 	control->priv->active = activated;
 }
@@ -230,17 +232,15 @@ bonobo_gtk_widget_from_x11_id (guint32 xid)
 
 	window = gdk_window_lookup (xid);
 	
-	if (! window) {
+	if (!window)
 		return NULL;
-	}
 
 	gdk_window_get_user_data(window, &data);
 
-	if (!GTK_IS_WIDGET (data)) {
+	if (!GTK_IS_WIDGET (data))
 		return NULL;
-	} else {
+	else
 		return GTK_WIDGET (data);
-	}
 }
 
 static gint
@@ -262,9 +262,8 @@ static void
 remove_destroy_idle (GtkWidget *socket,
 		     BonoboControl *control)
 {
-	if (control->priv->destroy_idle_id != 0) {
+	if (control->priv->destroy_idle_id != 0)
 		gtk_idle_remove (control->priv->destroy_idle_id);
-	}
 
 	control->priv->destroy_idle_id = 0;
 }
@@ -293,7 +292,6 @@ impl_Bonobo_Control_setWindowId (PortableServer_Servant  servant,
 	if (! local_socket) {
 		GtkWidget *old_plug;
 
-
 		old_plug            = control->priv->plug;
 
 		/* Create the new plug */
@@ -321,7 +319,9 @@ impl_Bonobo_Control_setWindowId (PortableServer_Servant  servant,
 		}
 
 		gtk_widget_show (control->priv->plug);
-		
+
+		control->priv->is_local = FALSE;
+
 	} else {
 		GtkWidget *socket_parent;
 
@@ -353,9 +353,9 @@ impl_Bonobo_Control_setWindowId (PortableServer_Servant  servant,
 
 static void
 impl_Bonobo_Control_setSize (PortableServer_Servant  servant,
-				   const CORBA_short       width,
-				   const CORBA_short       height,
-				   CORBA_Environment      *ev)
+			     const CORBA_short       width,
+			     const CORBA_short       height,
+			     CORBA_Environment      *ev)
 {
 	/*
 	 * Nothing.
@@ -368,13 +368,17 @@ impl_Bonobo_Control_setSize (PortableServer_Servant  servant,
 
 static void
 impl_Bonobo_Control_getDesiredSize (PortableServer_Servant  servant,
-				  CORBA_short            *desired_width,
-				  CORBA_short            *desired_height,
-				  CORBA_Environment      *ev)
+				    CORBA_short            *desired_width,
+				    CORBA_short            *desired_height,
+				    CORBA_Environment      *ev)
 {
-	/*
-	 * Nothing.
-	 */
+	BonoboControl *control = BONOBO_CONTROL (bonobo_object_from_servant (servant));
+	GtkRequisition requisition;
+
+	gtk_widget_size_request (control->priv->widget, &requisition);
+
+	*desired_width = requisition.width;
+	*desired_height = requisition.height;
 }
 
 static GtkStateType
@@ -432,6 +436,36 @@ impl_Bonobo_Control_getProperties (PortableServer_Servant  servant,
 	return bonobo_object_dup_ref (corba_propbag, ev);
 }
 
+static void
+process_events (PortableServer_Servant servant)
+{
+	BonoboControl *control =
+		BONOBO_CONTROL (bonobo_object_from_servant (servant));
+
+	g_return_if_fail (control != NULL);
+	g_return_if_fail (control->priv != NULL);
+
+	if (!control->priv->is_local) {
+		while (gtk_events_pending ())
+			gtk_main_iteration ();
+		gdk_flush ();
+	}
+}
+
+static void
+impl_Bonobo_Control_realize (PortableServer_Servant servant,
+			     CORBA_Environment     *ev)
+{
+	process_events (servant);
+}
+
+static void
+impl_Bonobo_Control_unrealize (PortableServer_Servant servant,
+			       CORBA_Environment     *ev)
+{
+	process_events (servant);
+}
+
 /**
  * bonobo_control_corba_object_create:
  * @object: the GtkObject that will wrap the CORBA object
@@ -641,6 +675,7 @@ bonobo_control_finalize (GtkObject *object)
 /**
  * bonobo_control_get_epv:
  *
+ * Returns: The EPV for the default BonoboControl implementation.  
  */
 POA_Bonobo_Control__epv *
 bonobo_control_get_epv (void)
@@ -656,6 +691,8 @@ bonobo_control_get_epv (void)
 	epv->setFrame       = impl_Bonobo_Control_setFrame;
 	epv->getDesiredSize = impl_Bonobo_Control_getDesiredSize;
 	epv->getProperties  = impl_Bonobo_Control_getProperties;
+	epv->realize        = impl_Bonobo_Control_realize;
+	epv->unrealize      = impl_Bonobo_Control_unrealize;
 
 	return epv;
 }
