@@ -191,6 +191,9 @@ impl_Bonobo_Embeddable_createView (PortableServer_Servant servant,
 	BonoboEmbeddable *embeddable = BONOBO_EMBEDDABLE (bonobo_object_from_servant (servant));
 	BonoboView       *view;
 	
+	if (embeddable->priv->view_factory == NULL)
+		return CORBA_OBJECT_NIL;
+
 	view = embeddable->priv->view_factory (
 		embeddable, view_frame,
 		embeddable->priv->view_factory_closure);
@@ -296,6 +299,8 @@ impl_Bonobo_Embeddable_createCanvasItem (PortableServer_Servant servant,
 
 /**
  * bonobo_embeddable_get_epv:
+ *
+ * Returns: The EPV for the default BonoboEmbeddable implementation. 
  */
 POA_Bonobo_Embeddable__epv *
 bonobo_embeddable_get_epv (void)
