@@ -17,7 +17,7 @@ struct _BonoboMonikerPrivate {
 	char          *name;
 };
 
-static BonoboObjectClass *bonobo_moniker_parent_class;
+static GtkObjectClass *bonobo_moniker_parent_class;
 
 POA_Bonobo_Moniker__vepv bonobo_moniker_vepv;
 
@@ -91,6 +91,14 @@ bonobo_moniker_get_name (BonoboMoniker *moniker,
 	return "";
 }
 
+/**
+ * bonobo_moniker_set_name:
+ * @moniker: the BonoboMoniker to configure.
+ * @name: new name for this moniker.
+ * @num_chars: number of characters in name to copy.
+ *
+ * This functions sets the moniker name in @moniker to be @name.
+ */
 void
 bonobo_moniker_set_name (BonoboMoniker *moniker,
 			 const char    *name,
@@ -167,6 +175,8 @@ impl_resolve (PortableServer_Servant       servant,
 
 /**
  * bonobo_moniker_get_epv:
+ *
+ * Returns: The EPV for the default Bonobo Moniker implementation.
  */
 POA_Bonobo_Moniker__epv *
 bonobo_moniker_get_epv (void)
@@ -204,7 +214,7 @@ bonobo_moniker_destroy (GtkObject *object)
 
 	g_free (moniker->priv);
 
-	GTK_OBJECT_CLASS (GTK_OBJECT (moniker)->klass)->destroy (object);
+	GTK_OBJECT_CLASS (bonobo_moniker_parent_class)->destroy (object);
 }
 
 static void
