@@ -16,6 +16,7 @@
 #include <bonobo/bonobo-main.h>
 #include <bonobo/bonobo-object.h>
 #include "Bonobo.h"
+#include "bonobo-running-context.h"
 #include "bonobo-object-directory.h"
 
 /* Assumptions made: sizeof(POA_interfacename) does not change between interfaces */
@@ -352,6 +353,7 @@ impl_Bonobo_Unknown_ref (PortableServer_Servant servant, CORBA_Environment *ev)
  *   This function returns a duplicated CORBA Object reference,
  * it also bumps the ref count on the object. This is ideal to
  * use in any method returning a Bonobo_Object in a CORBA impl.
+ * if object is CORBA_OBJECT_NIL it is returned unaffected.
  * 
  * Return value: duplicated & ref'd corba object reference.
  **/
@@ -361,9 +363,9 @@ bonobo_object_dup_ref (Bonobo_Unknown     object,
 {
 	CORBA_Environment tmpev, *rev;
 	Bonobo_Unknown    ans;
-	
-	g_return_val_if_fail (object != CORBA_OBJECT_NIL,
-			      CORBA_OBJECT_NIL);
+
+	if (object == CORBA_OBJECT_NIL)
+		return CORBA_OBJECT_NIL;
 
 	if (ev)
 		rev = ev;
@@ -388,8 +390,8 @@ bonobo_object_dup_ref (Bonobo_Unknown     object,
  * 
  *   This function returns releases a CORBA Object reference,
  * it also decrements the ref count on the bonobo object.
- * This is the converse of bonobo_object_release_ref
- * 
+ * This is the converse of bonobo_object_dup_ref we
+ * tolerate object == CORBA_OBJECT_NIL silently
  **/
 void
 bonobo_object_release_unref (Bonobo_Unknown     object,
@@ -397,7 +399,8 @@ bonobo_object_release_unref (Bonobo_Unknown     object,
 {
 	CORBA_Environment tmpev, *rev;
 
-	g_return_if_fail (object != CORBA_OBJECT_NIL);
+	if (object == CORBA_OBJECT_NIL)
+		return;
 
 	if (ev)
 		rev = ev;
@@ -511,7 +514,7 @@ bonobo_object_query_local_interface (BonoboObject *object,
 #else
 		    !strcmp (tryme->corba_objref->object_id, repo_id)
 #endif
-			){
+			) {
 			retval = tryme;
 			break;
 		}
@@ -579,6 +582,7 @@ bonobo_object_finalize_real (GtkObject *object)
 	CORBA_exception_init (&ev);
 
 	if (bonobo_object->corba_objref != CORBA_OBJECT_NIL) {
+		bonobo_running_context_remove_object (bonobo_object->corba_objref);
 		CORBA_Object_release (bonobo_object->corba_objref, &ev);
 		bonobo_object->corba_objref = CORBA_OBJECT_NIL;
 	}
@@ -618,8 +622,8 @@ bonobo_object_class_init (BonoboObjectClass *klass)
 				GTK_RUN_LAST,
 				object_class->type,
 				GTK_SIGNAL_OFFSET(BonoboObjectClass,system_exception),
-				gtk_marshal_NONE__POINTER,
-				GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
+				gtk_marshal_NONE__POINTER_POINTER,
+				GTK_TYPE_NONE, 2, GTK_TYPE_POINTER, GTK_TYPE_POINTER);
 	bonobo_object_signals [OBJECT_GONE] =
 		gtk_signal_new ("object_gone",
 				GTK_RUN_LAST,
@@ -796,6 +800,7 @@ bonobo_object_activate_servant (BonoboObject *object, void *servant)
 	if (o) {
 		object->corba_objref = o;
 		bonobo_object_bind_to_servant (object, servant);
+		bonobo_running_context_add_object (o);
 		return o;
 	} else
 		return CORBA_OBJECT_NIL;
@@ -834,7 +839,9 @@ bonobo_object_construct (BonoboObject *object, CORBA_Object corba_object)
  *
  * Adds the interfaces supported by @newobj to the list of interfaces
  * for @object.  This function adds the interfaces supported by
- * @newobj to the list of interfaces support by @object.
+ * @newobj to the list of interfaces support by @object. It should never
+ * be used when the object has been exposed to the world. This is a firm
+ * part of the contract.
  */
 void
 bonobo_object_add_interface (BonoboObject *object, BonoboObject *newobj)
@@ -935,16 +942,17 @@ bonobo_object_corba_objref (BonoboObject *object)
 void
 bonobo_object_check_env (BonoboObject *object, CORBA_Object obj, CORBA_Environment *ev)
 {
-	g_return_if_fail (BONOBO_IS_OBJECT (object));
 	g_return_if_fail (ev != NULL);
+	g_return_if_fail (BONOBO_IS_OBJECT (object));
 
 	if (ev->_major == CORBA_NO_EXCEPTION)
 		return;
 
-	if (ev->_major == CORBA_SYSTEM_EXCEPTION){
+	if (ev->_major == CORBA_SYSTEM_EXCEPTION)
 		gtk_signal_emit (
-			GTK_OBJECT (object), bonobo_object_signals [SYSTEM_EXCEPTION], obj, ev);
-	}
+			GTK_OBJECT (object),
+			bonobo_object_signals [SYSTEM_EXCEPTION],
+			obj, ev);
 }
 
 /**
