@@ -196,7 +196,8 @@ bonobo_print_client_new (Bonobo_Print corba_print)
 	g_return_val_if_fail (corba_print != CORBA_OBJECT_NIL, NULL);
 
 	if (!warned) {
-		g_warning ("The print client interface is horribly immature, use at your own risk");
+/* Still true, but to help Jon we'll keep quiet */
+/*		g_warning ("The print client interface is horribly immature, use at your own risk");*/
 		warned = TRUE;
 	}
 
@@ -247,9 +248,9 @@ bonobo_print_client_gtk_init (BonoboPrintClient *pbc)
 }
 
 /**
- * bonobo_property_bag_client_get_type:
+ * bonobo_print_client_get_type:
  *
- * Returns: The GtkType corresponding to the BonoboPropertyBagClient
+ * Returns: The GtkType corresponding to the BonoboPrintClient
  * class.
  */
 GtkType
