@@ -15,8 +15,6 @@
 
 BEGIN_GNOME_DECLS
 
-typedef struct _BonoboPropertyListener BonoboPropertyListener;
-
 #define BONOBO_PROPERTY_LISTENER_TYPE        (bonobo_property_listener_get_type ())
 #define BONOBO_PROPERTY_LISTENER(o)          (GTK_CHECK_CAST ((o), BONOBO_PROPERTY_LISTENER_TYPE, BonoboPropertyListener))
 #define BONOBO_PROPERTY_LISTENER_CLASS(k)    (GTK_CHECK_CLASS_CAST((k), BONOBO_PROPERTY_LISTENER_TYPE, BonoboPropertyListenerClass))
@@ -24,9 +22,9 @@ typedef struct _BonoboPropertyListener BonoboPropertyListener;
 #define BONOBO_IS_PROPERTY_LISTENER_CLASS(k) (GTK_CHECK_CLASS_TYPE ((k), BONOBO_PROPERTY_LISTENER_TYPE))
 
 
-struct _BonoboPropertyListener {
+typedef struct {
         BonoboObject		object;
-};
+} BonoboPropertyListener;
 
 typedef struct {
 	BonoboObjectClass parent_class;
