@@ -24,8 +24,8 @@
  *
  *      In order to do this, just call:
  * 
- *        bw = bonobo_widget_new_subdoc ("goad id of subdoc embddable",
- *                                             top_level_uihandler);
+ *        bw = bonobo_widget_new_subdoc ("moniker of subdoc embddable",
+ *                                        top_level_uicontainer);
  * 
  *      And then insert the 'bw' widget into the widget tree of your
  *      application like so:
@@ -40,11 +40,13 @@
  *      the work from about 5 lines to 1.  To embed a given control,
  *      just do:
  *
- *        bw = bonobo_widget_new_control ("goad id for control");
+ *        bw = bonobo_widget_new_control ("moniker for control");
  *        gtk_container_add (some_container, bw);
  *
  *      Ta da!
  *
+ *      NB. A simple moniker might look like 'file:/tmp/a.jpeg' or
+ *      OAFIID:GNOME_Evolution_Calendar_Control
  */
 
 #include <config.h>
@@ -54,6 +56,8 @@
 #include <bonobo/bonobo-main.h>
 #include <bonobo/bonobo-object.h>
 #include <bonobo/bonobo-widget.h>
+#include <bonobo/bonobo-exception.h>
+#include <bonobo/bonobo-moniker-util.h>
 
 struct _BonoboWidgetPrivate {
 
@@ -70,19 +74,36 @@ struct _BonoboWidgetPrivate {
 	BonoboItemContainer *container;
 	BonoboClientSite    *client_site;
 	BonoboViewFrame     *view_frame;
-	Bonobo_UIContainer   uih;
+	Bonobo_UIContainer   uic;
 };
 
 static BonoboWrapperClass *bonobo_widget_parent_class;
 
 static BonoboObjectClient *
-bonobo_widget_launch_component (const char *object_desc)
+bonobo_widget_launch_component (const char *moniker,
+				const char *if_name)
 {
+	Bonobo_Unknown corba_ref;
 	BonoboObjectClient *server;
+	CORBA_Environment ev;
 
-	server = bonobo_object_activate (object_desc, 0);
+	CORBA_exception_init (&ev);
+	corba_ref = bonobo_get_object (moniker, if_name, &ev);
 
-	return server;
+	if (BONOBO_EX (&ev)) {
+		char *txt;
+		g_warning ("Activation exception '%s'",
+			   (txt = bonobo_exception_get_text (&ev)));
+		g_free (txt);
+		server = CORBA_OBJECT_NIL;
+	}
+
+	CORBA_exception_free (&ev);
+
+	if (corba_ref == CORBA_OBJECT_NIL)
+		return NULL;
+
+	return bonobo_object_client_from_corba (corba_ref);
 }
 
 
@@ -94,14 +115,14 @@ bonobo_widget_launch_component (const char *object_desc)
 static BonoboWidget *
 bonobo_widget_construct_control_from_objref (BonoboWidget      *bw,
 					     Bonobo_Control     control,
-					     Bonobo_UIContainer uih)
+					     Bonobo_UIContainer uic)
 {
 	GtkWidget    *control_frame_widget;
 
 	/*
 	 * Create a local ControlFrame for it.
 	 */
-	bw->priv->control_frame = bonobo_control_frame_new (uih);
+	bw->priv->control_frame = bonobo_control_frame_new (uic);
 
 	bonobo_control_frame_bind_to_control (bw->priv->control_frame, control);
 
@@ -125,23 +146,24 @@ bonobo_widget_construct_control_from_objref (BonoboWidget      *bw,
 			   control_frame_widget);
 	gtk_widget_show (control_frame_widget);
 
-	if (uih != CORBA_OBJECT_NIL)
-		bw->priv->uih = bonobo_object_dup_ref (uih, NULL);
+	if (uic != CORBA_OBJECT_NIL)
+		bw->priv->uic = bonobo_object_dup_ref (uic, NULL);
 
 	return bw;
 }
 
 static BonoboWidget *
 bonobo_widget_construct_control (BonoboWidget      *bw,
-				 const char        *goad_id,
-				 Bonobo_UIContainer uih)
+				 const char        *moniker,
+				 Bonobo_UIContainer uic)
 {
 	Bonobo_Control control;
 
 	/*
 	 * Create the remote Control object.
 	 */
-	bw->priv->server = bonobo_widget_launch_component (goad_id);
+	bw->priv->server = bonobo_widget_launch_component (
+		moniker, "IDL:Bonobo/Control:1.0");
 	if (bw->priv->server == NULL) {
 		gtk_object_unref (GTK_OBJECT (bw));
 		return NULL;
@@ -149,12 +171,24 @@ bonobo_widget_construct_control (BonoboWidget      *bw,
 
 	control = bonobo_object_corba_objref (BONOBO_OBJECT (bw->priv->server));
 
-	return bonobo_widget_construct_control_from_objref (bw, control, uih);
+	return bonobo_widget_construct_control_from_objref (bw, control, uic);
 }
 
+/**
+ * bonobo_widget_new_control_from_objref:
+ * @control: A CORBA Object reference to an IDL:Bonobo/Control:1.0
+ * @uic: Bonobo_UIContainer for the launched object.
+ *
+ * This function is a simple wrapper for easily embedding controls
+ * into applications.  This function is used when you have already
+ * a CORBA object reference to an IDL:Bonobo/Control:1.0 (the
+ * @control) argument. 
+ *
+ * Returns: the @control wrapped as a #GtkWidget.
+ */
 GtkWidget *
 bonobo_widget_new_control_from_objref (Bonobo_Control     control,
-				       Bonobo_UIContainer uih)
+				       Bonobo_UIContainer uic)
 {
 	BonoboWidget *bw;
 
@@ -162,7 +196,7 @@ bonobo_widget_new_control_from_objref (Bonobo_Control     control,
 
 	bw = gtk_type_new (BONOBO_WIDGET_TYPE);
 
-	bw = bonobo_widget_construct_control_from_objref (bw, control, uih);
+	bw = bonobo_widget_construct_control_from_objref (bw, control, uic);
 
 	if (bw == NULL)
 		return NULL;
@@ -170,17 +204,31 @@ bonobo_widget_new_control_from_objref (Bonobo_Control     control,
 	return GTK_WIDGET (bw);
 }
 
+/**
+ * bonobo_widget_new_control:
+ * @moniker: A Moniker describing the object to be activated 
+ * @uic: Bonobo_UIContainer for the launched object.
+ *
+ * This function is a simple wrapper for easily embedding controls
+ * into applications.  It will launch the component identified by @id
+ * and will return it as a GtkWidget.
+ *
+ * FIXME: this function should really be using bonobo_get_object() instead
+ * of bonobo_activate_object() to launch the object.
+ *
+ * Returns: A #GtkWidget that is bound to the Bonobo Control. 
+ */
 GtkWidget *
-bonobo_widget_new_control (const char        *goad_id,
-			   Bonobo_UIContainer uih)
+bonobo_widget_new_control (const char        *moniker,
+			   Bonobo_UIContainer uic)
 {
 	BonoboWidget *bw;
 
-	g_return_val_if_fail (goad_id != NULL, NULL);
+	g_return_val_if_fail (moniker != NULL, NULL);
 
 	bw = gtk_type_new (BONOBO_WIDGET_TYPE);
 
-	bw = bonobo_widget_construct_control (bw, goad_id, uih);
+	bw = bonobo_widget_construct_control (bw, moniker, uic);
 
 	if (bw == NULL)
 		return NULL;
@@ -188,17 +236,25 @@ bonobo_widget_new_control (const char        *goad_id,
 		return GTK_WIDGET (bw);
 }
 
+/**
+ * bonobo_widget_get_control_frame:
+ * @bonobo_widget: a Bonobo Widget returned by one of the bonobo_widget_new() functions.
+ *
+ * Every IDL:Bonobo/Control:1.0 needs to be placed inside an
+ * IDL:Bonobo/ControlFrame:1.0.  This returns the BonoboControlFrame
+ * object that wraps the Control in the @bonobo_widget. 
+ * 
+ * Returns: The BonoboControlFrame associated with the @bonobo_widget
+ */
 BonoboControlFrame *
-bonobo_widget_get_control_frame (BonoboWidget *bw)
+bonobo_widget_get_control_frame (BonoboWidget *bonobo_widget)
 {
-	g_return_val_if_fail (bw != NULL, NULL);
-	g_return_val_if_fail (BONOBO_IS_WIDGET (bw), NULL);
+	g_return_val_if_fail (bonobo_widget != NULL, NULL);
+	g_return_val_if_fail (BONOBO_IS_WIDGET (bonobo_widget), NULL);
 
-	return bw->priv->control_frame;
+	return bonobo_widget->priv->control_frame;
 }
 
-
-
 
 /*
  *
@@ -207,8 +263,8 @@ bonobo_widget_get_control_frame (BonoboWidget *bw)
  */
 static BonoboWidget *
 bonobo_widget_create_subdoc_object (BonoboWidget      *bw,
-				    const char        *object_desc,
-				    Bonobo_UIContainer uih)
+				    const char        *moniker,
+				    Bonobo_UIContainer uic)
 {
 	GtkWidget *view_widget;
 	
@@ -218,7 +274,8 @@ bonobo_widget_create_subdoc_object (BonoboWidget      *bw,
 	 */
 	bw->priv->container = bonobo_item_container_new ();
 
-	bw->priv->server = bonobo_widget_launch_component (object_desc);
+	bw->priv->server = bonobo_widget_launch_component (
+		moniker, "IDL:Bonobo/Embeddable:1.0");
 	if (bw->priv->server == NULL)
 		return NULL;
 	
@@ -245,7 +302,7 @@ bonobo_widget_create_subdoc_object (BonoboWidget      *bw,
 	/*
 	 * Now create a new view for the remote object.
 	 */
-	bw->priv->view_frame = bonobo_client_site_new_view (bw->priv->client_site, uih);
+	bw->priv->view_frame = bonobo_client_site_new_view (bw->priv->client_site, uic);
 
 	/*
 	 * Add the view frame.
@@ -254,26 +311,42 @@ bonobo_widget_create_subdoc_object (BonoboWidget      *bw,
 	gtk_container_add (GTK_CONTAINER (bw), view_widget);
 	gtk_widget_show (view_widget);
 
-	if (uih != CORBA_OBJECT_NIL)
-		bw->priv->uih = bonobo_object_dup_ref (uih, NULL);
+	if (uic != CORBA_OBJECT_NIL)
+		bw->priv->uic = bonobo_object_dup_ref (uic, NULL);
 	
 	return bw;
 }
 
+/**
+ * bonobo_widget_new_subdoc:
+ * @moniker: A moniker description of the Object to be activated.
+ * @uic: Bonobo_UIContainer for the launched object.
+ *
+ * This function is a simple wrapper for easily embedding documents
+ * into applications.  It will launch the component identified by @id
+ * and will return it as a GtkWidget.
+ *
+ * This will launch a single view of the embeddable activated by @moniker.
+ *
+ * FIXME: this function should really be using bonobo_get_object() instead
+ * of bonobo_activate_object() to launch the object.
+ *
+ * Returns: A #GtkWidget that is bound to the Bonobo Control. 
+ */
 GtkWidget *
-bonobo_widget_new_subdoc (const char        *object_desc,
-			  Bonobo_UIContainer uih)
+bonobo_widget_new_subdoc (const char        *moniker,
+			  Bonobo_UIContainer uic)
 {
 	BonoboWidget *bw;
 
-	g_return_val_if_fail (object_desc != NULL, NULL);
+	g_return_val_if_fail (moniker != NULL, NULL);
 
 	bw = gtk_type_new (BONOBO_WIDGET_TYPE);
 
 	if (bw == NULL)
 		return NULL;
 
-	if (!bonobo_widget_create_subdoc_object (bw, object_desc, uih)) {
+	if (!bonobo_widget_create_subdoc_object (bw, moniker, uic)) {
 		gtk_object_destroy (GTK_OBJECT (bw));
 		return NULL;
 	}
@@ -283,41 +356,74 @@ bonobo_widget_new_subdoc (const char        *object_desc,
 	return GTK_WIDGET (bw);
 }
 
-
+/**
+ * bonobo_widget_get_container
+ * @bonobo_widget: the #BonoboWidget to query.
+ *
+ * This operation is only valid for BonoboWidgets that were created
+ * by the bonobo_widget_new_subdoc(). 
+ *
+ * Returns: the BonoboItemContainer associated with this @bonobo_widget
+ */
 BonoboItemContainer *
-bonobo_widget_get_container (BonoboWidget *bw)
+bonobo_widget_get_container (BonoboWidget *bonobo_widget)
 {
-	g_return_val_if_fail (bw != NULL, NULL);
-	g_return_val_if_fail (BONOBO_IS_WIDGET (bw), NULL);
+	g_return_val_if_fail (bonobo_widget != NULL, NULL);
+	g_return_val_if_fail (BONOBO_IS_WIDGET (bonobo_widget), NULL);
 
-	return bw->priv->container;
+	return bonobo_widget->priv->container;
 }
 
+/**
+ * bonobo_widget_get_client_site:
+ * @bonobo_widget: the #BonoboWidget to query.
+ *
+ * This operation is only valid for BonoboWidgets that were created
+ * by the bonobo_widget_new_subdoc(). 
+ *
+ * Returns: the #BonoboClientSite associated with this @bonobo_widget
+ */
 BonoboClientSite *
-bonobo_widget_get_client_site (BonoboWidget *bw)
+bonobo_widget_get_client_site (BonoboWidget *bonobo_widget)
 {
-	g_return_val_if_fail (bw != NULL, NULL);
-	g_return_val_if_fail (BONOBO_IS_WIDGET (bw), NULL);
+	g_return_val_if_fail (bonobo_widget != NULL, NULL);
+	g_return_val_if_fail (BONOBO_IS_WIDGET (bonobo_widget), NULL);
 
-	return bw->priv->client_site;
+	return bonobo_widget->priv->client_site;
 }
 
+/**
+ * bonobo_widget_get_view_frame:
+ * @bonobo_widget: the #BonoboWidget to query.
+ *
+ * This operation is only valid for BonoboWidgets that were created
+ * by the bonobo_widget_new_subdoc(). 
+ *
+ * Returns: The #BonoboViewFrame associated with this @bonobo_widget.
+ */
 BonoboViewFrame *
-bonobo_widget_get_view_frame (BonoboWidget *bw)
+bonobo_widget_get_view_frame (BonoboWidget *bonobo_widget)
 {
-	g_return_val_if_fail (bw != NULL, NULL);
-	g_return_val_if_fail (BONOBO_IS_WIDGET (bw), NULL);
+	g_return_val_if_fail (bonobo_widget != NULL, NULL);
+	g_return_val_if_fail (BONOBO_IS_WIDGET (bonobo_widget), NULL);
 
-	return bw->priv->view_frame;
+	return bonobo_widget->priv->view_frame;
 }
 
+/**
+ * bonobo_widget_get_uih:
+ * @bonobo_widget: the #BonoboWidget to query.
+ *
+ * Returns: the CORBA object reference to the Bonobo_UIContainer
+ * associated with the @bonobo_widget.
+ */
 Bonobo_UIContainer
-bonobo_widget_get_uih (BonoboWidget *bw)
+bonobo_widget_get_uih (BonoboWidget *bonobo_widget)
 {
-	g_return_val_if_fail (bw != NULL, NULL);
-	g_return_val_if_fail (BONOBO_IS_WIDGET (bw), NULL);
+	g_return_val_if_fail (bonobo_widget != NULL, NULL);
+	g_return_val_if_fail (BONOBO_IS_WIDGET (bonobo_widget), NULL);
 
-	return bw->priv->uih;
+	return bonobo_widget->priv->uic;
 }
 
 
@@ -327,13 +433,21 @@ bonobo_widget_get_uih (BonoboWidget *bw)
  * Generic (non-control/subdoc specific) BonoboWidget stuff.
  *
  */
-BonoboObjectClient *
-bonobo_widget_get_server (BonoboWidget *bw)
-{
-	g_return_val_if_fail (bw != NULL, NULL);
-	g_return_val_if_fail (BONOBO_IS_WIDGET (bw), NULL);
 
-	return bw->priv->server;
+/**
+ * bonobo_widget_get_server:
+ * @bonobo_widget: the #BonoboWidget to query.
+ *
+ * Returns: The BonoboObjectClient (a wrapper for the CORBA object
+ * reference) to the object that this BonoboWidget is wrapping. 
+ */ 
+BonoboObjectClient *
+bonobo_widget_get_server (BonoboWidget *bonobo_widget)
+{
+	g_return_val_if_fail (bonobo_widget != NULL, NULL);
+	g_return_val_if_fail (BONOBO_IS_WIDGET (bonobo_widget), NULL);
+
+	return bonobo_widget->priv->server;
 }
 
 static void
@@ -350,8 +464,8 @@ bonobo_widget_destroy (GtkObject *object)
 		bonobo_object_unref (BONOBO_OBJECT (priv->client_site));
 	if (priv->view_frame)
 		bonobo_object_unref (BONOBO_OBJECT (priv->view_frame));
-	if (priv->uih != CORBA_OBJECT_NIL)
-		bonobo_object_release_unref (priv->uih, NULL);
+	if (priv->uic != CORBA_OBJECT_NIL)
+		bonobo_object_release_unref (priv->uic, NULL);
 
 	g_free (priv);
 	GTK_OBJECT_CLASS (bonobo_widget_parent_class)->destroy (object);
@@ -446,6 +560,29 @@ bonobo_widget_get_type (void)
 	return type;
 }
 
+/**
+ * bonobo_widget_set_property:
+ * @control: A #BonoboWidget that represents an IDL:Bonobo/Control:1.0
+ * @first_prop: first property name to set.
+ *
+ * This is a utility function used to set a number of properties
+ * in the Bonobo Control in @control.
+ *
+ * This function takes a variable list of arguments that must be NULL
+ * terminated.  Arguments come in tuples: a string (for the argument
+ * name) and the data type that is to be transfered.  The
+ * implementation of the actual setting of the PropertyBag values is
+ * done by the bonobo_property_bag_client_setv() function).
+ *
+ * FIXME: This function is error prone because it depends on the
+ * client and the server to agree on the data types to be sent.  If
+ * the server arguments change the data type, this routine will not
+ * be able to cope gracefully with this condition.
+ *
+ * This only works for BonoboWidgets that represent controls (ie,
+ * that were returned by bonobo_widget_new_control_from_objref() or
+ * bonobo_widget_new_control().
+ */
 void
 bonobo_widget_set_property (BonoboWidget      *control,
 			    const char        *first_prop, ...)
@@ -484,6 +621,29 @@ bonobo_widget_set_property (BonoboWidget      *control,
 }
 
 
+/**
+ * bonobo_widget_get_property:
+ * @control: A #BonoboWidget that represents an IDL:Bonobo/Control:1.0
+ * @first_prop: first property name to set.
+ *
+ * This is a utility function used to get a number of properties
+ * in the Bonobo Control in @control.
+ *
+ * This function takes a variable list of arguments that must be NULL
+ * terminated.  Arguments come in tuples: a string (for the argument
+ * name) and a pointer where the data will be stored.  The
+ * implementation of the actual setting of the PropertyBag values is
+ * done by the bonobo_property_bag_client_setv() function).
+ *
+ * FIXME: This function is error prone because it depends on the
+ * client and the server to agree on the data types to be sent.  If
+ * the server arguments change the data type, this routine will not
+ * be able to cope gracefully with this condition.
+ *
+ * This only works for BonoboWidgets that represent controls (ie,
+ * that were returned by bonobo_widget_new_control_from_objref() or
+ * bonobo_widget_new_control().
+ */
 void
 bonobo_widget_get_property (BonoboWidget      *control,
 			    const char        *first_prop, ...)
