@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/**
+/*
  * bonobo-win.c: The Bonobo Window implementation.
  *
  * Author:
@@ -40,8 +40,12 @@
 
 GtkWindowClass *bonobo_window_parent_class = NULL;
 
+typedef struct _StateUpdate StateUpdate;
+
 struct _BonoboWindowPrivate {
-	BonoboWindow     *win;
+	BonoboWindow  *win;
+
+	BonoboObject  *container;
 
 	int            frozen;
 
@@ -68,6 +72,8 @@ struct _BonoboWindowPrivate {
 	GtkStatusbar  *main_status;
 
 	GtkWidget     *client_area;
+
+	GSList        *state_updates;
 };
 
 typedef enum {
@@ -151,7 +157,7 @@ info_dump_fn (BonoboUIXml *tree, BonoboUINode *node)
  		} else
 			fprintf (stderr, " no associated widget\n");
 	} else
-		fprintf (stderr, " very wierd no data on node '%p'\n", node);
+		fprintf (stderr, " very weird no data on node '%p'\n", node);
 }
 
 /* We need to map the shell to the item */
@@ -266,6 +272,36 @@ win_component_get (BonoboWindowPrivate *priv, const char *name)
 	return component;
 }
 
+static WinComponent *
+win_component_get_by_ref (BonoboWindowPrivate *priv, CORBA_Object obj)
+{
+	GSList *l;
+	WinComponent *component = NULL;
+	CORBA_Environment ev;
+
+	g_return_val_if_fail (priv != NULL, NULL);
+	g_return_val_if_fail (obj != CORBA_OBJECT_NIL, NULL);
+
+	CORBA_exception_init (&ev);
+
+	for (l = priv->components; l; l = l->next) {
+		gboolean equiv;
+		component = l->data;
+
+		equiv = CORBA_Object_is_equivalent (component->object, obj, &ev);
+
+		if (BONOBO_EX (&ev)) { /* Something very badly wrong */
+			component = NULL;
+			break;
+		} else if (equiv)
+			break;
+	}
+
+	CORBA_exception_free (&ev);
+
+	return component;
+}
+
 static Bonobo_Unknown
 win_component_objref (BonoboWindowPrivate *priv, const char *name)
 {
@@ -330,13 +366,35 @@ win_component_destroy (BonoboWindowPrivate *priv, WinComponent *component)
 }
 
 void
-bonobo_window_register_component (BonoboWindow     *win,
-			       const char    *name,
-			       Bonobo_Unknown component)
+bonobo_window_deregister_dead_components (BonoboWindow *win)
+{
+	WinComponent      *component;
+	GSList            *l, *next;
+	CORBA_Environment  ev;
+
+	g_return_if_fail (BONOBO_IS_WINDOW (win));
+
+	for (l = win->priv->components; l; l = next) {
+		next = l->next;
+
+		component = l->data;
+		CORBA_exception_init (&ev);
+
+		if (CORBA_Object_non_existent (component->object, &ev))
+			bonobo_window_deregister_component (win, component->name);
+
+		CORBA_exception_free (&ev);
+	}
+}
+
+void
+bonobo_window_register_component (BonoboWindow  *win,
+				  const char    *name,
+				  Bonobo_Unknown component)
 {
 	WinComponent *wincomp;
 
-	g_return_if_fail (BONOBO_IS_WIN (win));
+	g_return_if_fail (BONOBO_IS_WINDOW (win));
 
 	if ((wincomp = win_component_get (win->priv, name))) {
 		if (wincomp->object != CORBA_OBJECT_NIL)
@@ -350,12 +408,12 @@ bonobo_window_register_component (BonoboWindow     *win,
 }
 
 void
-bonobo_window_deregister_component (BonoboWindow     *win,
-				 const char    *name)
+bonobo_window_deregister_component (BonoboWindow *win,
+				    const char   *name)
 {
 	WinComponent *component;
 
-	g_return_if_fail (BONOBO_IS_WIN (win));
+	g_return_if_fail (BONOBO_IS_WINDOW (win));
 
 	if ((component = win_component_get (win->priv, name))) {
 		bonobo_window_xml_rm (win, "/", component->name);
@@ -365,6 +423,22 @@ bonobo_window_deregister_component (BonoboWindow     *win,
 			   "component '%s'", name);
 }
 
+void
+bonobo_window_deregister_component_by_ref (BonoboWindow  *win,
+					   Bonobo_Unknown ref)
+{
+	WinComponent *component;
+
+	g_return_if_fail (BONOBO_IS_WINDOW (win));
+
+	if ((component = win_component_get_by_ref (win->priv, ref))) {
+		bonobo_window_xml_rm (win, "/", component->name);
+		win_component_destroy (win->priv, component);
+	} else
+		g_warning ("Attempting to deregister non-registered "
+			   "component");
+}
+
 static gboolean
 widget_is_special (GtkWidget *widget)
 {
@@ -466,15 +540,73 @@ placeholder_sync (BonoboWindowPrivate *priv,
 		bonobo_ui_node_free_string (txt);
 	}
 
-	if (!bonobo_ui_node_children (node))
-		show = FALSE;
-
 	if (show)
 		gtk_widget_show (widget);	
 	else
 		gtk_widget_hide (widget);
 }
 
+static void
+hide_all_widgets (BonoboWindowPrivate *priv,
+		  BonoboUINode *node)
+{
+	NodeInfo *info;
+	BonoboUINode *child;
+	
+	info = bonobo_ui_xml_get_data (priv->tree, node);
+	if (info->widget)
+		gtk_widget_hide (info->widget);
+	
+	for (child = bonobo_ui_node_children (node);
+	     child != NULL;
+	     child = bonobo_ui_node_next (child))
+		hide_all_widgets (priv, child);
+}
+
+static gboolean
+contains_visible_widget (BonoboWindowPrivate *priv,
+			 BonoboUINode *node)
+{
+	BonoboUINode *child;
+	NodeInfo *info;
+	
+	for (child = bonobo_ui_node_children (node);
+	     child != NULL;
+	     child = bonobo_ui_node_next (child)) {
+		info = bonobo_ui_xml_get_data (priv->tree, child);
+		if (info->widget && GTK_WIDGET_VISIBLE (info->widget))
+			return TRUE;
+		if (contains_visible_widget (priv, child))
+			return TRUE;
+	}
+
+	return FALSE;
+}
+
+static void
+hide_placeholder_if_empty_or_hidden (BonoboWindowPrivate *priv,
+				     BonoboUINode *node)
+{
+	NodeInfo *info;
+	char *txt;
+	gboolean hide_placeholder_and_contents;
+	gboolean has_visible_separator;
+
+	txt = bonobo_ui_node_get_attr (node, "hidden");
+	hide_placeholder_and_contents = txt && atoi (txt);
+	bonobo_ui_node_free_string (txt);
+
+	info = bonobo_ui_xml_get_data (priv->tree, node);
+	has_visible_separator = info && info->widget
+		&& GTK_WIDGET_VISIBLE (info->widget);
+
+	if (hide_placeholder_and_contents)
+		hide_all_widgets (priv, node);
+	else if (has_visible_separator
+		 && !contains_visible_widget (priv, node))
+		gtk_widget_hide (info->widget);
+}
+
 typedef void       (*SyncStateFn)   (BonoboWindowPrivate *priv,
 				     BonoboUINode     *node,
 				     GtkWidget        *widget,
@@ -598,13 +730,13 @@ sync_generic_widgets (BonoboWindowPrivate *priv,
 					printf ("not dirty & not same, but has no widget\n");
 #endif
 				} else {
-					printf ("Bonobo-WARNING **: non dirty node, but widget mismatch "
-						"a: '%s:%s', b: '%s:%s' '%p'\n\n",
-						bonobo_ui_node_get_name (a),
-						bonobo_ui_node_get_attr (a, "name"),
-						bn ? bonobo_ui_node_get_name (bn) : "NULL",
-						bn ? bonobo_ui_node_get_attr (bn, "name") : "NULL",
-						info->widget);
+					g_warning ("non dirty node, but widget mismatch "
+						   "a: '%s:%s', b: '%s:%s' '%p'",
+						   bonobo_ui_node_get_name (a),
+						   bonobo_ui_node_get_attr (a, "name"),
+						   bn ? bonobo_ui_node_get_name (bn) : "NULL",
+						   bn ? bonobo_ui_node_get_attr (bn, "name") : "NULL",
+						   info->widget);
 				}
 			}
 #ifdef WIDGET_SYNC_DEBUG
@@ -614,10 +746,12 @@ sync_generic_widgets (BonoboWindowPrivate *priv,
 			(*pos)++;
 		}
 
-		if (bonobo_ui_node_has_name (a, "placeholder"))
+		if (bonobo_ui_node_has_name (a, "placeholder")) {
 			sync_generic_widgets (priv, bonobo_ui_node_children (a),
 					      parent, &nextb, pos, sync_state,
 					      build_widget, make_placeholder);
+			hide_placeholder_if_empty_or_hidden (priv, a);
+		}
 
 		a = bonobo_ui_node_next (a);
 	}
@@ -642,29 +776,17 @@ check_excess_widgets (BonoboWindowPrivate *priv, GList *wptr)
 				continue;
 			
 			if (!warned++)
-				printf ("Bonono-Warning **: Excess widgets at the "
-					"end of the container; wierd\n\n");
+				g_warning ("Excess widgets at the "
+					   "end of the container; weird");
 
 			node = widget_get_node (b->data);
-			printf ("Widget type '%s' with node: '%s'\n",
-				gtk_type_name (GTK_OBJECT (b->data)->klass->type),
-				node ? bonobo_ui_xml_make_path (node) : "NULL");
+			g_message ("Widget type '%s' with node: '%s'",
+				   gtk_type_name (GTK_OBJECT (b->data)->klass->type),
+				   node ? bonobo_ui_xml_make_path (node) : "NULL");
 		}
 	}
 }
 
-static void
-cmd_set_dirty (BonoboWindowPrivate *priv, BonoboUINode *cmd_node)
-{
-	BonoboUIXmlData *data;
-
-	if (!cmd_node)
-		return;
-
-	data = bonobo_ui_xml_get_data (priv->tree, cmd_node);
-	data->dirty = TRUE;
-}
-
 /*
  * Returns: TRUE if hiddenness changed.
  */
@@ -696,65 +818,159 @@ do_show_hide (GtkWidget *widget, BonoboUINode *node)
 	return changed;
 }
 
-static void
-widget_set_state (GtkWidget *widget, BonoboUINode *node)
+struct _StateUpdate {
+	GtkWidget *widget;
+	char      *state;
+};
+
+/* Update the state later, but other aspects of the widget right now.
+ * It's dangerous to update the state now because we can reenter if we
+ * do that.
+ */
+static StateUpdate *
+state_update_new (GtkWidget *widget, BonoboUINode *node)
 {
-	char *txt;
+	char *hidden, *sensitive;
+	StateUpdate *su;
 
-	if ((txt = bonobo_ui_node_get_attr (node, "state"))) {
-		if (BONOBO_IS_UI_TOOLBAR_ITEM (widget))
-			bonobo_ui_toolbar_item_set_state (
-				BONOBO_UI_TOOLBAR_ITEM (widget), txt);
+	g_return_val_if_fail (node != NULL, NULL);
+	g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
 
-		else if (GTK_IS_CHECK_MENU_ITEM (widget)) {
-#ifdef STATE_SYNC_DEBUG
-			g_warning ("Setting check menu item '%p' to '%s'",
-				   widget, txt);
-#endif
-			gtk_check_menu_item_set_active (
-				GTK_CHECK_MENU_ITEM (widget), 
-				atoi (txt));
-		} else
-			g_warning ("TESTME: strange, setting "
-				   "state '%s' on wierd object '%s'",
-				   txt, gtk_type_name (GTK_OBJECT (
-					   widget)->klass->type));
+	hidden = bonobo_ui_node_get_attr (node, "hidden");
+	if (hidden && atoi (hidden))
+		gtk_widget_hide (widget);
+	else
+		gtk_widget_show (widget);
+	bonobo_ui_node_free_string (hidden);
 
-		bonobo_ui_node_free_string (txt);
-	}
+	sensitive = bonobo_ui_node_get_attr (node, "sensitive");
+	if (sensitive)
+		gtk_widget_set_sensitive (widget, atoi (sensitive));
+	bonobo_ui_node_free_string (sensitive);
 
-	if ((txt = bonobo_ui_node_get_attr (node, "sensitive"))) {
-		gtk_widget_set_sensitive (widget, atoi (txt));
-		bonobo_ui_node_free_string (txt);
-	}
+	su = g_new0 (StateUpdate, 1);
+	su->widget = widget;
+	gtk_widget_ref (su->widget);
+	su->state = bonobo_ui_node_get_attr (node, "state");
 
-	do_show_hide (widget, node);
+	return su;
 }
 
 static void
-update_cmd_state (BonoboWindowPrivate *priv, BonoboUINode *search,
-		  BonoboUINode *state, const char *search_id)
+state_update_destroy (StateUpdate *su)
 {
-	BonoboUINode    *l;
+	if (su) {
+		gtk_widget_unref (su->widget);
+		bonobo_ui_node_free_string (su->state);
+
+		g_free (su);
+	}
+}
+
+static void
+state_update_exec (StateUpdate *su)
+{
+	g_return_if_fail (su != NULL);
+	g_return_if_fail (su->widget != NULL);
+
+	if (su->state) {
+		if (BONOBO_IS_UI_TOOLBAR_ITEM (su->widget))
+			bonobo_ui_toolbar_item_set_state (
+				BONOBO_UI_TOOLBAR_ITEM (su->widget), su->state);
+
+		else if (GTK_IS_CHECK_MENU_ITEM (su->widget)) {
+#ifdef STATE_SYNC_DEBUG
+			g_warning ("Setting check menu item '%p' to '%s'",
+				   su->widget, su->state);
+#endif
+			gtk_check_menu_item_set_active (
+				GTK_CHECK_MENU_ITEM (su->widget), 
+				atoi (su->state));
+		} else
+			g_warning ("TESTME: strange, setting "
+				   "state '%s' on weird object '%s'",
+				   su->state, gtk_type_name (GTK_OBJECT (
+					   su->widget)->klass->type));
+	}
+}
+
+static void
+widget_set_state (GtkWidget *widget, BonoboUINode *node)
+{
+	StateUpdate *su;
+
+	if (!widget)
+		return;
+
+	su = state_update_new (widget, node);
+	
+	state_update_exec (su);
+
+	state_update_destroy (su);
+}
+
+static void
+widget_queue_state (BonoboWindowPrivate *priv,
+		    GtkWidget           *widget,
+		    BonoboUINode        *node)
+{
+	StateUpdate *su = state_update_new (widget, node);
+	
+	priv->state_updates = g_slist_prepend (
+		priv->state_updates, su);
+}
+
+static GSList *
+find_widgets_for_command (BonoboWindowPrivate *priv,
+			  GSList              *list,
+			  BonoboUINode        *search,
+			  BonoboUINode        *state,
+			  const char          *search_id)
+{
+	BonoboUINode *l;
 	char *id = node_get_id (search);
 
-	g_return_if_fail (search_id != NULL);
-
 /*	printf ("Update cmd state if %s == %s on node '%s'\n", search_id, id,
-	bonobo_ui_node_get_name (search));*/
+	bonobo_ui_xml_make_path (search));*/
 
 	if (id && !strcmp (search_id, id)) { /* Sync its state */
 		NodeInfo *info = bonobo_ui_xml_get_data (priv->tree, search);
 
 		if (info->widget)
-			widget_set_state (info->widget, state);
+			list = g_slist_prepend (
+				list, state_update_new (info->widget, state));
 	}
 
 	g_free (id);
 
 	for (l = bonobo_ui_node_children (search); l;
              l = bonobo_ui_node_next (l))
-		update_cmd_state (priv, l, state, search_id);
+		list = find_widgets_for_command (priv, list, l, state, search_id);
+
+	return list;
+}
+
+static void
+update_cmd_state (BonoboWindowPrivate *priv, BonoboUINode *search,
+		  BonoboUINode *state, const char *search_id)
+{
+	GSList *updates, *l;
+
+	g_return_if_fail (search_id != NULL);
+
+/*	printf ("Update cmd state on %s from node '%s'\n", search_id,
+	bonobo_ui_xml_make_path (state));*/
+
+	updates = find_widgets_for_command (
+		priv, NULL, priv->tree->root, state, search_id);
+
+	for (l = updates; l; l = l->next)
+		state_update_exec (l->data);
+
+	for (l = updates; l; l = l->next)
+		state_update_destroy (l->data);
+
+	g_slist_free (updates);
 }
 
 static void
@@ -891,7 +1107,7 @@ cmd_get_menu_pixmap (BonoboUINode     *node,
 	return NULL;
 }
 
-/**
+/*
  * set_cmd_attr:
  *   Syncs cmd / widgets on events [ event flag set ]
  *   or helps evil people who set state on menu /
@@ -914,8 +1130,8 @@ set_cmd_attr (BonoboWindowPrivate *priv,
 	if (!(cmd_node = cmd_get_node (priv, node))) { /* A non cmd widget */
 		NodeInfo *info = bonobo_ui_xml_get_data (priv->tree, node);
 
-		widget_set_state (info->widget, node);
 		bonobo_ui_node_set_attr (node, prop, value);
+		widget_set_state (info->widget, node);
 		return;
 	}
 
@@ -962,11 +1178,14 @@ real_emit_ui_event (BonoboWindowPrivate *priv, const char *component_name,
 		Bonobo_UIComponent_uiEvent (
 			component, id, type, new_state, &ev);
 
-		if (ev._major != CORBA_NO_EXCEPTION) {
-			/* FIXME: so if it is a sys exception do we de-merge ? */
-			g_warning ("Exception emitting state change to %d '%s' '%s'",
-				   type, id, new_state);
-		}
+		if (priv->container)
+			bonobo_object_check_env (
+				priv->container, component, &ev);
+
+		if (ev._major != CORBA_NO_EXCEPTION)
+			g_warning ("Exception emitting state change to %d '%s' '%s'"
+				   "major %d, %s",
+				   type, id, new_state, ev._major, ev._repo_id);
 		
 		CORBA_exception_free (&ev);
 	} else
@@ -1000,7 +1219,7 @@ static GnomeDockItem *
 get_dock_item (BonoboWindowPrivate *priv,
 	       const char       *dockname)
 {
-	guint          dummy;
+	guint dummy;
 	
 	return gnome_dock_get_item_by_name (priv->dock,
 					    dockname,
@@ -1250,11 +1469,14 @@ real_exec_verb (BonoboWindowPrivate *priv,
 		Bonobo_UIComponent_execVerb (
 			component, verb, &ev);
 
-		if (ev._major != CORBA_NO_EXCEPTION) {
-			/* FIXME: so if it is a sys exception do we de-merge ? */
-			g_warning ("Exception executing verb '%s' on '%s'",
-				   verb, component_name);
-		}
+		if (priv->container)
+			bonobo_object_check_env (
+				priv->container, component, &ev);
+
+		if (ev._major != CORBA_NO_EXCEPTION)
+			g_warning ("Exception executing verb '%s' '%s'"
+				   "major %d, %s",
+				   verb, component_name, ev._major, ev._repo_id);
 		
 		CORBA_exception_free (&ev);
 	} else
@@ -1280,7 +1502,7 @@ exec_verb_cb (GtkWidget *item, BonoboWindowPrivate  *priv)
 		return FALSE;
 
 	if (!data->id) {
-		g_warning ("Wierd; no ID on verb '%s'", verb);
+		g_warning ("Weird; no ID on verb '%s'", verb);
 		bonobo_ui_node_free_string (verb);
 		return FALSE;
 	}
@@ -1298,6 +1520,7 @@ menu_toggle_emit_ui_event (GtkCheckMenuItem *item, BonoboWindowPrivate *priv)
 	BonoboUINode     *node = widget_get_node (GTK_WIDGET (item));
 	char             *id, *state;
 	BonoboUIXmlData  *data;
+	char             *component_id;
 
 	g_return_val_if_fail (node != NULL, FALSE);
 
@@ -1311,13 +1534,17 @@ menu_toggle_emit_ui_event (GtkCheckMenuItem *item, BonoboWindowPrivate *priv)
 		state = "1";
 	else
 		state = "0";
-	
+
+	component_id = g_strdup (data->id);
+
+	/* This could invoke a CORBA method that might de-register the component */
 	set_cmd_attr (priv, node, "state", state, TRUE);
 
-	real_emit_ui_event (priv, data->id, id,
+	real_emit_ui_event (priv, component_id, id,
 			    Bonobo_UIComponent_STATE_CHANGED,
 			    state);
 
+	g_free (component_id);
 	g_free (id);
 
 	return FALSE;
@@ -1326,11 +1553,12 @@ menu_toggle_emit_ui_event (GtkCheckMenuItem *item, BonoboWindowPrivate *priv)
 static gint
 win_item_emit_ui_event (BonoboUIToolbarItem *item,
 			const char          *state,
-			BonoboWindowPrivate    *priv)
+			BonoboWindowPrivate *priv)
 {
 	BonoboUINode     *node = widget_get_node (GTK_WIDGET (item));
 	char             *id;
 	BonoboUIXmlData  *data;
+	char             *component_id;
 
 	g_return_val_if_fail (node != NULL, FALSE);
 
@@ -1340,11 +1568,16 @@ win_item_emit_ui_event (BonoboUIToolbarItem *item,
 	data = bonobo_ui_xml_get_data (NULL, node);
 	g_return_val_if_fail (data != NULL, FALSE);
 
+	component_id = g_strdup (data->id);
+
+	/* This could invoke a CORBA method that might de-register the component */
 	set_cmd_attr (priv, node, "state", state, TRUE);
 
-	real_emit_ui_event (priv, data->id, id,
+	real_emit_ui_event (priv, component_id, id,
 			    Bonobo_UIComponent_STATE_CHANGED,
 			    state);
+
+	g_free (component_id);
 	g_free (id);
 
 	return FALSE;
@@ -1551,6 +1784,9 @@ menu_build_item (BonoboWindowPrivate *priv,
 
 			bonobo_ui_node_free_string (txt);
 		}
+
+		if (!menu_widget)
+			return NULL;
 			
 		gtk_signal_connect (GTK_OBJECT (menu_widget),
 				    "select",
@@ -1562,7 +1798,10 @@ menu_build_item (BonoboWindowPrivate *priv,
 				    GTK_SIGNAL_FUNC (remove_hint_from_statusbar),
 				    priv);
 	}
-		   
+
+	if (!menu_widget)
+		return NULL;	
+	   
 	if (bonobo_ui_node_has_name (node, "submenu")) {
 		GtkMenuShell *shell;
 		GtkMenu      *menu;
@@ -1633,31 +1872,35 @@ menu_sync_state (BonoboWindowPrivate *priv, BonoboUINode *node,
 	if ((hidden    = bonobo_ui_node_get_attr (node, "hidden")) ||
 	    (sensitive = bonobo_ui_node_get_attr (node, "sensitive")) ||
 	    (state     = bonobo_ui_node_get_attr (node, "state"))) {
-		if (cmd_node && !warned++) {
-			char *txt;
-			g_warning ("FIXME: We have an attribute '%s' at '%s' breaking "
-				   "cmd/widget separation, please fix",
-				   hidden?"hidden":((sensitive)?"sensitive":((state)?"state":"error")),
-				   (txt = bonobo_ui_xml_make_path (node)));
-			g_free (txt);
+		if (cmd_node) {
+			if (!warned++) {
+				char *txt;
+				g_warning ("FIXME: We have an attribute '%s' at '%s' breaking "
+					   "cmd/widget separation, please fix",
+					   hidden?"hidden":((sensitive)?"sensitive":((state)?"state":"error")),
+					   (txt = bonobo_ui_xml_make_path (node)));
+				g_free (txt);
+			}
 			if (hidden)
-				set_cmd_attr (priv, cmd_node, "hidden", hidden, FALSE);
+				set_cmd_attr (priv, node, "hidden", hidden, FALSE);
 			if (sensitive)
-				set_cmd_attr (priv, cmd_node, "sensitive", sensitive, FALSE);
+				set_cmd_attr (priv, node, "sensitive", sensitive, FALSE);
 			if (state && (txt = bonobo_ui_node_get_attr (node, "type"))) {
-				set_cmd_attr (priv, cmd_node, "state", state, FALSE);
+				set_cmd_attr (priv, node, "state", state, FALSE);
 				bonobo_ui_node_free_string (txt);
 			}
 		}
-		widget_set_state (widget, node);
 	}
 	bonobo_ui_node_free_string (state);
 	bonobo_ui_node_free_string (sensitive);
 	bonobo_ui_node_free_string (hidden);
 
 	if (bonobo_ui_node_has_name (node, "placeholder") ||
-	    bonobo_ui_node_has_name (node, "separator"))
+	    bonobo_ui_node_has_name (node, "separator")) {
+		
+		widget_set_state (widget, node);
 		return;
+	}
 
 	if (bonobo_ui_node_has_name (node, "submenu")) {
 		menu_widget = get_item_widget (widget);
@@ -1772,6 +2015,8 @@ menu_sync_state (BonoboWindowPrivate *priv, BonoboUINode *node,
 					    key, mods,
 					    GTK_ACCEL_VISIBLE);
 	}
+
+	widget_queue_state (priv, menu_widget, cmd_node != NULL ? cmd_node : node);
 }
 
 static GtkWidget *
@@ -2090,20 +2335,20 @@ toolbar_sync_state (BonoboWindowPrivate *priv, BonoboUINode *node,
 	if ((hidden    = bonobo_ui_node_get_attr (node, "hidden")) ||
 	    (sensitive = bonobo_ui_node_get_attr (node, "sensitive")) ||
 	    (state     = bonobo_ui_node_get_attr (node, "state"))) {
-		if (cmd_node && !warned++) {
-			g_warning ("FIXME: We have an attribute '%s' at '%s' breaking "
-				   "cmd/widget separation, please fix",
-				   hidden?"hidden":((sensitive)?"sensitive":((state)?"state":"error")),
-				   bonobo_ui_xml_make_path (node));
+		if (cmd_node) {
+			if (!warned++) {
+				g_warning ("FIXME: We have an attribute '%s' at '%s' breaking "
+					   "cmd/widget separation, please fix",
+					   hidden?"hidden":((sensitive)?"sensitive":((state)?"state":"error")),
+					   bonobo_ui_xml_make_path (node));
+			}
 			if (hidden)
-				set_cmd_attr (priv, cmd_node, "hidden", hidden, FALSE);
+				set_cmd_attr (priv, node, "hidden", hidden, FALSE);
 			if (sensitive)
-				set_cmd_attr (priv, cmd_node, "sensitive", sensitive, FALSE);
+				set_cmd_attr (priv, node, "sensitive", sensitive, FALSE);
 			if (state)
-				set_cmd_attr (priv, cmd_node, "state", state, FALSE);
-			cmd_set_dirty (priv, cmd_node);
+				set_cmd_attr (priv, node, "state", state, FALSE);
 		}
-		widget_set_state (widget, node);
 	}
 	bonobo_ui_node_free_string (state);
 	bonobo_ui_node_free_string (sensitive);
@@ -2199,15 +2444,27 @@ toolbar_sync_state (BonoboWindowPrivate *priv, BonoboUINode *node,
 		g_free (decoded_txt);
 		bonobo_ui_node_free_string (txt);
 	}
+
+	widget_queue_state (priv, widget, cmd_node != NULL ? cmd_node : node);
 }
 
 static BonoboUIToolbarStyle
 parse_look (const char *look)
 {
-	if (!look || !strcmp (look, "both"))
-		return BONOBO_UI_TOOLBAR_STYLE_ICONS_AND_TEXT;
-	else
-		return BONOBO_UI_TOOLBAR_STYLE_ICONS_ONLY;
+	if (look) {
+		if (!strcmp (look, "both"))
+			return BONOBO_UI_TOOLBAR_STYLE_ICONS_AND_TEXT;
+
+		if (!strcmp (look, "icon"))
+			return BONOBO_UI_TOOLBAR_STYLE_ICONS_ONLY;
+
+		if (!strcmp (look, "text"))
+			return BONOBO_UI_TOOLBAR_STYLE_PRIORITY_TEXT;	
+	}
+
+	return gnome_preferences_get_toolbar_labels ()
+		? BONOBO_UI_TOOLBAR_STYLE_ICONS_AND_TEXT
+		: BONOBO_UI_TOOLBAR_STYLE_ICONS_ONLY;
 }
 
 static void
@@ -2686,6 +2943,20 @@ update_popups (BonoboWindowPrivate *priv, BonoboUINode *node)
 	}
 }
 
+static void
+process_state_updates (BonoboWindowPrivate *priv)
+{
+	while (priv->state_updates) {
+		StateUpdate *su = priv->state_updates->data;
+
+		priv->state_updates = g_slist_remove (
+			priv->state_updates, su);
+
+		state_update_exec (su);
+		state_update_destroy (su);
+	}
+}
+
 static void
 update_widgets (BonoboWindowPrivate *priv)
 {
@@ -2723,16 +2994,25 @@ update_widgets (BonoboWindowPrivate *priv)
 
 	update_commands_state (priv);
 
+	process_state_updates (priv);
+
 /*	bonobo_window_dump (priv->win, "after update");*/
 }
 
 static void
 popup_remove (BonoboWindowPrivate *priv,
-	      WinPopup         *popup)
+	      WinPopup            *popup)
 {
+	BonoboUINode *node;
+
 	g_return_if_fail (priv != NULL);
 	g_return_if_fail (popup != NULL);
 
+	gtk_signal_disconnect_by_data (GTK_OBJECT (popup->menu), popup);
+
+	node = bonobo_ui_xml_get_path (priv->tree, popup->path);
+	prune_widget_info (priv, node, TRUE);
+
 	priv->popups = g_slist_remove (
 		priv->popups, popup);
 	
@@ -2742,12 +3022,12 @@ popup_remove (BonoboWindowPrivate *priv,
 
 void
 bonobo_window_remove_popup (BonoboWindow     *win,
-			 const char    *path)
+			    const char    *path)
 {
 	GSList *l, *next;
 
 	g_return_if_fail (path != NULL);
-	g_return_if_fail (BONOBO_IS_WIN (win));
+	g_return_if_fail (BONOBO_IS_WINDOW (win));
 	g_return_if_fail (win->priv != NULL);
 
 	for (l = win->priv->popups; l; l = next) {
@@ -2766,13 +3046,13 @@ popup_destroy (GtkObject *menu, WinPopup *popup)
 		GTK_OBJECT (menu), BONOBO_WINDOW_PRIV_KEY);
 
 	g_return_if_fail (priv != NULL);
-	bonobo_window_remove_popup (priv->win, popup->path);
+	popup_remove (priv, popup);
 }
 
 void
-bonobo_window_add_popup (BonoboWindow     *win,
-		      GtkMenu       *menu,
-		      const char    *path)
+bonobo_window_add_popup (BonoboWindow *win,
+			 GtkMenu      *menu,
+			 const char   *path)
 {
 	WinPopup     *popup;
 	BonoboUINode    *node;
@@ -2780,7 +3060,7 @@ bonobo_window_add_popup (BonoboWindow     *win,
 
 	g_return_if_fail (path != NULL);
 	g_return_if_fail (GTK_IS_MENU (menu));
-	g_return_if_fail (BONOBO_IS_WIN (win));
+	g_return_if_fail (BONOBO_IS_WINDOW (win));
 
 	bonobo_window_remove_popup (win, path);
 
@@ -2800,21 +3080,57 @@ bonobo_window_add_popup (BonoboWindow     *win,
 	node = bonobo_ui_xml_get_path_wildcard (
 		win->priv->tree, path, &wildcard);
 
-	if (node) {
-		BonoboUIXmlData *data;
-
-		data = bonobo_ui_xml_get_data (win->priv->tree, node);
-		g_return_if_fail (data != NULL);
-
-		data->dirty = TRUE;
-	}
+	if (node)
+		bonobo_ui_xml_set_dirty (win->priv->tree, node);
 
 	update_widgets (win->priv);
 }
 
+GList *
+bonobo_window_deregister_get_component_names (BonoboWindow *win)
+{
+	WinComponent *component;
+	GSList *l;
+	GList *retval;
+
+	g_return_val_if_fail (BONOBO_IS_WINDOW (win), NULL);
+
+	retval = NULL;
+
+	for (l = win->priv->components; l; l = l->next) {
+		component = l->data;
+	
+		retval = g_list_prepend (retval, component->name);
+	}
+
+	return retval;
+}
+
+
+Bonobo_Unknown
+bonobo_window_component_get (BonoboWindow *win,
+			      const char  *name)
+{
+	WinComponent *component;
+	GSList *l;
+
+	g_return_val_if_fail (BONOBO_IS_WINDOW (win), CORBA_OBJECT_NIL);
+	g_return_val_if_fail (name != NULL, CORBA_OBJECT_NIL);
+		
+	for (l = win->priv->components; l; l = l->next) {
+		component = l->data;
+		
+		if (!strcmp (component->name, name))
+			return component->object;
+	}
+
+	return CORBA_OBJECT_NIL;
+}
+
+
 void
 bonobo_window_set_contents (BonoboWindow *win,
-			 GtkWidget *contents)
+			    GtkWidget    *contents)
 {
 	g_return_if_fail (win != NULL);
 	g_return_if_fail (win->priv != NULL);
@@ -2857,8 +3173,15 @@ radio_group_destroy (gpointer	key,
 static void
 destroy_priv (BonoboWindowPrivate *priv)
 {
+	if (priv->container)
+		gtk_signal_disconnect_by_data (
+			GTK_OBJECT (priv->container), priv->win);
+
 	priv->win = NULL;
 
+	while (priv->popups)
+		popup_remove (priv, priv->popups->data);
+
 	gtk_object_unref (GTK_OBJECT (priv->tree));
 	priv->tree = NULL;
 
@@ -2880,9 +3203,6 @@ destroy_priv (BonoboWindowPrivate *priv)
 
 	while (priv->components)
 		win_component_destroy (priv, priv->components->data);
-
-	while (priv->popups)
-		popup_remove (priv, priv->popups->data);
 	
 	g_free (priv);
 }
@@ -2902,14 +3222,14 @@ bonobo_window_finalize (GtkObject *object)
 
 char *
 bonobo_window_xml_get (BonoboWindow  *win,
-		    const char *path,
-		    gboolean    node_only)
+		       const char *path,
+		       gboolean    node_only)
 {
  	char *str;
  	BonoboUINode    *node;
   	CORBA_char *ret;
   
-  	g_return_val_if_fail (BONOBO_IS_WIN (win), NULL);
+  	g_return_val_if_fail (BONOBO_IS_WINDOW (win), NULL);
   
   	node = bonobo_ui_xml_get_path (win->priv->tree, path);
   	if (!node)
@@ -2924,12 +3244,12 @@ bonobo_window_xml_get (BonoboWindow  *win,
 
 gboolean
 bonobo_window_xml_node_exists (BonoboWindow  *win,
-			    const char *path)
+			       const char *path)
 {
 	BonoboUINode *node;
 	gboolean      wildcard;
 
-	g_return_val_if_fail (BONOBO_IS_WIN (win), FALSE);
+	g_return_val_if_fail (BONOBO_IS_WINDOW (win), FALSE);
 
 	node = bonobo_ui_xml_get_path_wildcard (
 		win->priv->tree, path, &wildcard);
@@ -2943,14 +3263,15 @@ bonobo_window_xml_node_exists (BonoboWindow  *win,
 
 BonoboUIXmlError
 bonobo_window_object_set (BonoboWindow  *win,
-		       const char *path,
-		       Bonobo_Unknown object,
-		       CORBA_Environment *ev)
+			  const char    *path,
+			  Bonobo_Unknown object,
+			  CORBA_Environment *ev)
 {
 	BonoboUINode   *node;
 	NodeInfo  *info;
 
-	g_return_val_if_fail (BONOBO_IS_WIN (win), BONOBO_UI_XML_BAD_PARAM);
+	g_return_val_if_fail (BONOBO_IS_WINDOW (win),
+			      BONOBO_UI_XML_BAD_PARAM);
 
 	node = bonobo_ui_xml_get_path (win->priv->tree, path);
 	if (!node)
@@ -2983,16 +3304,16 @@ bonobo_window_object_set (BonoboWindow  *win,
 }
 
 BonoboUIXmlError
-bonobo_window_object_get (BonoboWindow  *win,
-		       const char *path,
-		       Bonobo_Unknown *object,
-		       CORBA_Environment *ev)
+bonobo_window_object_get (BonoboWindow   *win,
+			  const char     *path,
+			  Bonobo_Unknown *object,
+			  CORBA_Environment *ev)
 {
 	BonoboUINode *node;
 	NodeInfo *info;
 
 	g_return_val_if_fail (object != NULL, BONOBO_UI_XML_BAD_PARAM);
-	g_return_val_if_fail (BONOBO_IS_WIN (win), BONOBO_UI_XML_BAD_PARAM);
+	g_return_val_if_fail (BONOBO_IS_WINDOW (win), BONOBO_UI_XML_BAD_PARAM);
 
 	*object = CORBA_OBJECT_NIL;
 
@@ -3009,10 +3330,10 @@ bonobo_window_object_get (BonoboWindow  *win,
 }
 
 BonoboUIXmlError
-bonobo_window_xml_merge_tree (BonoboWindow  *win,
-			   const char *path,
-			   BonoboUINode    *tree,
-			   const char *component)
+bonobo_window_xml_merge_tree (BonoboWindow *win,
+			      const char   *path,
+			      BonoboUINode *tree,
+			      const char   *component)
 {
 	BonoboUIXmlError err;
 	
@@ -3040,6 +3361,7 @@ bonobo_window_xml_merge_tree (BonoboWindow  *win,
 		err = bonobo_ui_xml_merge (
 			win->priv->tree, path, bonobo_ui_node_children (tree),
 			win_component_cmp_name (win->priv, component));
+		bonobo_ui_node_free (tree);
 	} else
 		err = bonobo_ui_xml_merge (
 			win->priv->tree, path, tree,
@@ -3053,10 +3375,10 @@ bonobo_window_xml_merge_tree (BonoboWindow  *win,
 }
 
 BonoboUIXmlError
-bonobo_window_xml_merge (BonoboWindow  *win,
-		      const char *path,
-		      const char *xml,
-		      const char *component)
+bonobo_window_xml_merge (BonoboWindow *win,
+			 const char   *path,
+			 const char   *xml,
+			 const char   *component)
 {
 	BonoboUIXmlError err;
 	BonoboUINode *node;
@@ -3079,9 +3401,9 @@ bonobo_window_xml_merge (BonoboWindow  *win,
 }
 
 BonoboUIXmlError
-bonobo_window_xml_rm (BonoboWindow  *win,
-		   const char *path,
-		   const char *by_component)
+bonobo_window_xml_rm (BonoboWindow *win,
+		      const char   *path,
+		      const char   *by_component)
 {
 	BonoboUIXmlError err;
 
@@ -3101,7 +3423,7 @@ bonobo_window_xml_rm (BonoboWindow  *win,
 void
 bonobo_window_freeze (BonoboWindow *win)
 {
-	g_return_if_fail (BONOBO_IS_WIN (win));
+	g_return_if_fail (BONOBO_IS_WINDOW (win));
 
 	win->priv->frozen++;
 }
@@ -3109,7 +3431,7 @@ bonobo_window_freeze (BonoboWindow *win)
 void
 bonobo_window_thaw (BonoboWindow *win)
 {
-	g_return_if_fail (BONOBO_IS_WIN (win));
+	g_return_if_fail (BONOBO_IS_WINDOW (win));
 	
 	if (--win->priv->frozen <= 0) {
 		update_widgets (win->priv);
@@ -3118,10 +3440,10 @@ bonobo_window_thaw (BonoboWindow *win)
 }
 
 void
-bonobo_window_dump (BonoboWindow  *win,
-		 const char *msg)
+bonobo_window_dump (BonoboWindow *win,
+		    const char   *msg)
 {
-	g_return_if_fail (BONOBO_IS_WIN (win));
+	g_return_if_fail (BONOBO_IS_WINDOW (win));
 
 	fprintf (stderr, "Bonobo Win '%s': frozen '%d'\n",
 		 win->priv->name, win->priv->frozen);
@@ -3134,16 +3456,16 @@ bonobo_window_dump (BonoboWindow  *win,
 GtkAccelGroup *
 bonobo_window_get_accel_group (BonoboWindow *win)
 {
-	g_return_val_if_fail (BONOBO_IS_WIN (win), NULL);
+	g_return_val_if_fail (BONOBO_IS_WINDOW (win), NULL);
 
 	return win->priv->accel_group;
 }
 
 
 static gint
-bonobo_window_binding_handle (GtkWidget        *widget,
-			   GdkEventKey      *event,
-			   BonoboWindowPrivate *priv)
+bonobo_window_binding_handle (GtkWidget           *widget,
+			      GdkEventKey         *event,
+			      BonoboWindowPrivate *priv)
 {
 	Binding  lookup, *binding;
 
@@ -3177,13 +3499,13 @@ construct_priv (BonoboWindow *win)
 
 	priv = g_new0 (BonoboWindowPrivate, 1);
 
-	priv->win    = win;	
+	priv->win = win;	
 
 	/* Keybindings; the gtk_binding stuff is just too evil */
 	gtk_signal_connect (GTK_OBJECT (win), "key_press_event",
 			    (GtkSignalFunc) bonobo_window_binding_handle, priv);
 
-	priv->dock   = GNOME_DOCK (gnome_dock_new ());
+	priv->dock = GNOME_DOCK (gnome_dock_new ());
 	gtk_container_add (GTK_CONTAINER (win),
 			   GTK_WIDGET    (priv->dock));
 
@@ -3283,13 +3605,34 @@ bonobo_window_init (BonoboWindow *win)
 	win->priv = construct_priv (win);
 }
 
+static void
+blank_container (BonoboUIContainer *container, BonoboWindow *win)
+{
+	if (win->priv)
+		win->priv->container = NULL;
+}
+
+void
+bonobo_window_set_ui_container (BonoboWindow *win,
+				BonoboObject *container)
+{
+	g_return_if_fail (BONOBO_IS_WINDOW (win));
+	g_return_if_fail (!container ||
+			  BONOBO_IS_OBJECT (container));
+
+	win->priv->container = container;
+	if (container)
+		gtk_signal_connect (GTK_OBJECT (container), "destroy",
+				    (GtkSignalFunc) blank_container, win);
+}
+
 void
 bonobo_window_set_name (BonoboWindow  *win,
-		     const char *win_name)
+			const char *win_name)
 {
 	BonoboWindowPrivate *priv;
 
-	g_return_if_fail (BONOBO_IS_WIN (win));
+	g_return_if_fail (BONOBO_IS_WINDOW (win));
 
 	priv = win->priv;
 
@@ -3308,7 +3651,7 @@ bonobo_window_set_name (BonoboWindow  *win,
 char *
 bonobo_window_get_name (BonoboWindow *win)
 {
-	g_return_val_if_fail (BONOBO_IS_WIN (win), NULL);
+	g_return_val_if_fail (BONOBO_IS_WINDOW (win), NULL);
 	g_return_val_if_fail (win->priv != NULL, NULL);
 
 	if (win->priv->name)
@@ -3318,11 +3661,11 @@ bonobo_window_get_name (BonoboWindow *win)
 }
 
 GtkWidget *
-bonobo_window_construct (BonoboWindow  *win,
-		      const char *win_name,
-		      const char *title)
+bonobo_window_construct (BonoboWindow *win,
+			 const char   *win_name,
+			 const char   *title)
 {
-	g_return_val_if_fail (BONOBO_IS_WIN (win), NULL);
+	g_return_val_if_fail (BONOBO_IS_WINDOW (win), NULL);
 
 	bonobo_window_set_name (win, win_name);
 
