@@ -261,11 +261,11 @@ real_buffer_create (BonoboObjectClient *server)
 }
 
 static Buffer *
-real_buffer_create_embeddable (const char *component_goad_id)
+real_buffer_create_embeddable (const char *component_oaf_iid)
 {
 	BonoboObjectClient *server;
 
-	server = bonobo_object_activate (component_goad_id, 0);
+	server = bonobo_object_activate (component_oaf_iid, 0);
 
 	if (!server) {
 		g_warning ("Launching component failed");
@@ -276,13 +276,13 @@ real_buffer_create_embeddable (const char *component_goad_id)
 }
 
 static Buffer *
-real_buffer_create_control (const char *component_goad_id)
+real_buffer_create_control (const char *component_oaf_iid)
 {
 	BonoboObjectClient *server;
 
-	server = gshell_control_wrapper (component_goad_id);
+	server = gshell_control_wrapper (component_oaf_iid);
 	if (!server) {
-		g_warning("Launching component failed");
+		g_warning ("Launching component failed");
 		return NULL;
 	}
 
@@ -290,7 +290,7 @@ real_buffer_create_control (const char *component_goad_id)
 }
 
 Buffer *
-buffer_create (const char *component_goad_id)
+buffer_create (const char *component_oaf_iid)
 {
 	Buffer *retval;
         OAF_ServerInfoList *servers;
@@ -299,15 +299,15 @@ buffer_create (const char *component_goad_id)
 
         CORBA_exception_init (&ev);
 	query = g_strdup_printf ("(iid == '%s') AND repo_ids.has('IDL:Bonobo/Embeddable:1.0')",
-				 component_goad_id);
+				 component_oaf_iid);
         servers = oaf_query (query, NULL, &ev);
         g_free (query);
         CORBA_exception_free (&ev);
 
 	if ((servers != NULL) && (servers->_length > 0))
-		retval = real_buffer_create_embeddable (component_goad_id);
+		retval = real_buffer_create_embeddable (component_oaf_iid);
 	else
-		retval = real_buffer_create_control (component_goad_id);
+		retval = real_buffer_create_control (component_oaf_iid);
 
 	CORBA_free (servers);
 
@@ -315,9 +315,9 @@ buffer_create (const char *component_goad_id)
 }
 
 Buffer *
-buffer_create_for_control (const char *component_goad_id)
+buffer_create_for_control (const char *component_oaf_iid)
 {
-	return real_buffer_create_control (component_goad_id);
+	return real_buffer_create_control (component_oaf_iid);
 }
 
 static GtkWidget *
@@ -329,7 +329,7 @@ find_view_frame_pos (Frame *frame, BonoboViewFrame *view_frame, gint *pos)
 
 	l = GTK_BOX(frame->vbox)->children;
 	
-	while (l && (o = GTK_OBJECT(((GtkBoxChild *)l->data)->widget)) && 
+	while (l && (o = GTK_OBJECT (((GtkBoxChild *)l->data)->widget)) && 
 	       (gtk_object_get_data (o, "gshell:view_frame") != view_frame)) {
 		i++; l = l->next; 
 	}
