@@ -41,6 +41,7 @@ static EFSResult  ib1_node_stat  (EFSNode *node, EFSStat *stat);
 static EFSResult  ib1_erase      (EFSDir *dir, const char *path); 
 static EFSResult  ib1_rename     (EFSDir *dir, const char *old_path, 
 				  const char *new_path);
+static gboolean   ib1_node_equal (EFSNode *node1, EFSNode *node2);
 
 EFSFileOps file_ops_ib1 = {
 	ib1_node_open,
@@ -55,7 +56,8 @@ EFSFileOps file_ops_ib1 = {
 	ib1_type_get,
 	ib1_node_stat,
 	ib1_erase,
-	ib1_rename
+	ib1_rename,
+	ib1_node_equal
 };
 
 static IB1DirEntry   root_de = { GUINT32_TO_LE(IB1_ROOT_INODE), 
@@ -531,7 +533,7 @@ ib1_file_read (EFSFile *efs_file, gpointer buf, gint32 count,
 	node = NODEP(ce, file->inode);
 	size = GUINT32_FROM_LE(node->size);
 	if ((efs_file->pos+count) > size) count = size - efs_file->pos;
-	if (count <= 0) return EFS_ERR_EOF; /* eof */
+	if (count <= 0) return EFS_ERR_OK; /* eof */
 
 	if (efs_file->pos < IB1_IDATA_LEN) { /* read from inode */
 		mb = MIN ((IB1_IDATA_LEN-efs_file->pos), count);
@@ -663,7 +665,7 @@ ib1_dir_read  (EFSDir *efs_dir, EFSDirEntry *res_de)
 		}
 	}
 
-	return EFS_ERR_EOF;
+	return EFS_ERR_NOENT; /* eof */
 }
 
 /*
@@ -685,7 +687,7 @@ ib1_file_trunc (EFSFile *efs_file, guint32 size)
 	node = NODEP(ce, file->inode);
 	CLOCK(ce);
 	fs = GUINT32_FROM_LE(node->size);
-	if (fs <= size) return EFS_ERR_EOF;
+	if (fs <= size) return EFS_ERR_OK; /* eof */
 
 	if (size <= IB1_IDATA_LEN) block = 0;
 	else block = ((size-IB1_IDATA_LEN)+507)/508;
@@ -824,4 +826,21 @@ ib1_rename (EFSDir *efs_dir, const char *old_path, const char *new_path)
 	return EFS_ERR_OK;
 }
 
+/*
+ * ib1_node_equal:
+ *
+ * see efs_node_equal().
+ */
+
+static gboolean
+ib1_node_equal (EFSNode *node1, EFSNode *node2)
+{
+	IB1File *n1 = (IB1File *)node1;
+	IB1File *n2 = (IB1File *)node2;
+
+	if (n1->inode == n2->inode) return TRUE;
+
+	return FALSE;
+}
+
 
