@@ -9,11 +9,7 @@
 
 #include <config.h>
 #include <gnome.h>
-#ifdef USING_OAF
 #include <liboaf/liboaf.h>
-#else
-#include <libgnorba/gnorba.h>
-#endif
 #include <bonobo.h>
 #include "Echo.h"
 
@@ -21,27 +17,13 @@ static void
 init_bonobo (int argc, char *argv [])
 {
 	CORBA_ORB orb;
-#ifndef USING_OAF
-	CORBA_Environment ev;
 
-	CORBA_exception_init (&ev);
-	
-	gnome_CORBA_init_with_popt_table (
-		"echo-client", "1.0",
-		&argc, argv, NULL, 0, NULL,
-		GNORBA_INIT_SERVER_FUNC, &ev);
-
-	orb = gnome_CORBA_ORB ();
-
-	CORBA_exception_free (&ev);
-#else
         gnome_init_with_popt_table (
 		"echo-client", "1.0",
 		argc, argv,
 		oaf_popt_options, 0, NULL); 
 
 	orb = oaf_init (argc, argv);
-#endif
 
 	if (!bonobo_init (orb, CORBA_OBJECT_NIL,
 			  CORBA_OBJECT_NIL))
@@ -63,12 +45,8 @@ main (int argc, char *argv [])
 
 	init_bonobo (argc, argv);
 
-#if USING_OAF
 	obj_id = "OAFIID:demo_echo:fe45dab2-ae27-45e9-943d-34a49eefca96";
-		
-#else
-	obj_id = "GOADID:demo_echo";
-#endif
+
 	server = bonobo_object_activate (obj_id, 0);
 
 	if (!server) {
@@ -90,7 +68,7 @@ main (int argc, char *argv [])
 
 	CORBA_exception_free (&ev);
 
-	bonobo_object_unref (server);
+	bonobo_object_unref (BONOBO_OBJECT (server));
 	
 	return 0;
 }
